# -*- test-case-name: buildbot.test.test_words -*-

import sys, os, tempfile

from twisted.internet import defer
from twisted.trial import unittest
dr = unittest.deferredResult
from twisted.python import log, components
#log.startLogging(sys.stderr)


#from buildbot import interfaces
from buildbot.status import words, builder
#from buildbot.status import progress, client # NEEDS COVERAGE
from buildbot.test.test_status import MyBuilder, MyBuild

class MyBotMaster:
    builderNames = []

class MyIRC(words.IRC):
    def _botMsg(self, msg):
        self.parent.messages.append(msg)

class IRC(unittest.TestCase):
    def setUp(self):
        self.builder = MyBuilder("builder1")

    def makeBuild(self, number, results):
        return MyBuild(self.builder, number, results)

    def failUnlessIn(self, substring, string):
        self.failUnless(string.find(substring) != -1)

    def getBuildbotURL(self):
        return "BUILDBOT_URL"

    def testBuildSucceed(self):
        # a first successful build does not trigger a praise
        irc = MyIRC('localhost', 'trialbot', ["#buildbot", ])
        irc.parent = self
        irc.status = self
        self.messages = []

        b1 = self.makeBuild(3, builder.SUCCESS)
        b1.blamelist = ["bob"]

        irc.buildFinished("builder1", b1, b1.results)
        self.failUnless(len(self.messages) == 0)

    def testBuildBlame(self):
        irc = MyIRC('localhost', 'trialbot', ["#buildbot", ])
        irc.parent = self
        irc.status = self
        self.messages = []

        b1 = self.makeBuild(3, builder.FAILURE)
        b1.blamelist = ["bob"]

        irc.buildFinished("builder1", b1, b1.results)
        self.failUnless(len(self.messages) == 1)
        m = self.messages.pop()
        m2 = words._DEFAULT_BLAMES[0] % "bob"
        self.failUnless(m.startswith(m2))

    def testBuildBlamePraise(self):
        irc = MyIRC('localhost', 'trialbot', ["#buildbot", ])
        irc.parent = self
        irc.status = self
        self.messages = []

        b1 = self.makeBuild(3, builder.FAILURE)
        b1.blamelist = ["bob"]

        irc.buildFinished("builder1", b1, b1.results)
        self.failUnless(len(self.messages) == 1)
        m = self.messages.pop()
        m2 = words._DEFAULT_BLAMES[0] % "bob"
        self.failUnless(m.startswith(m2))

        b2 = self.makeBuild(4, builder.SUCCESS)
        b2.blamelist = ["john"]

        irc.buildFinished("builder1", b2, b2.results)
        self.failUnless(len(self.messages) == 1)
        m = self.messages.pop()
        m2 = words._DEFAULT_PRAISES[0] % "john"
        self.failUnless(m.startswith(m2))

class Subscription(unittest.TestCase):
    # test subscription and categories

    def setUp(self):
        self.tempdir = tempfile.mkdtemp()

    def tearDown(self):
        os.system("rm -r %s" % self.tempdir)

    def _botMsg(self, msg):
        self.parent.messages.append(msg)

    def _setupIRC(self, categories=None):
        self.irc = MyIRC('localhost', 'trialbot', ["#buildbot", ],
            categories=categories)
        self.irc.parent = self # make messages work
        self.messages = []

    def testUnsubscribedAdded(self):
        self._setupIRC()
        status = builder.Status(MyBotMaster(), self.tempdir)
        bers = status.builderAdded("builder1", self.tempdir)
        self.assertEquals(len(bers.watchers), 0)
        self.failUnless(len(self.messages) == 0)
        status.builderRemoved("builder1")
        
    def testSubscribedBuilderAdded(self):
        self._setupIRC()
        status = builder.Status(MyBotMaster(), self.tempdir)

        status.subscribe(self.irc)

        bers = status.builderAdded("builder1", self.tempdir)
        self.failUnless(len(self.messages) == 1)
        m = self.messages.pop()
        self.assertEquals(m, "builder builder1 added")

        status.unsubscribe(self.irc)
        bers = status.builderAdded("builder2", self.tempdir)
        self.failUnless(len(self.messages) == 0)

        status.builderRemoved("builder1")
        status.builderRemoved("builder2")

    def _finishBuild(self, bers, result, blamelist):
        bs = bers.newBuild()
        bers.buildStarted(bs)
        bs.results = result
        bs.blamelist = blamelist
        bers._buildFinished(bs)

    def testSubscribedBuildFinished(self):
        self._setupIRC()
        status = builder.Status(MyBotMaster(), self.tempdir)

        status.subscribe(self.irc)
        self.assertEquals(status.watchers, [self.irc])

        # add a builder and make sure we get notified
        bers = status.builderAdded("builder1", self.tempdir)
        self.assertEquals(len(bers.watchers), 1)
        self.assertEquals(len(self.messages), 1)
        m = self.messages.pop()
        self.assertEquals(m, "builder builder1 added")

        # finish a failing build from bob
        self._finishBuild(bers, builder.FAILURE, ["bob"])

        self.assertEquals(len(self.messages), 1)
        m = self.messages.pop()
        m2 = words._DEFAULT_BLAMES[0] % "bob"
        self.failUnless(m.startswith(m2))

        status.unsubscribe(self.irc)
        status.builderRemoved("builder1")

    def testSubscribedCategoryBuildFinished(self):
        self._setupIRC(categories=["debug", ])
        status = builder.Status(MyBotMaster(), self.tempdir)

        status.subscribe(self.irc)
        self.assertEquals(status.watchers, [self.irc])

        # add a builder and make sure we do not subscribe because of no cat
        bers = status.builderAdded("builder1", self.tempdir)
        self.assertEquals(len(bers.watchers), 0)
        self.assertEquals(len(self.messages), 0)

        # finish a failing build and see no message
        self._finishBuild(bers, builder.FAILURE, ["bob"])
        self.assertEquals(len(self.messages), 0)

        # add a builder and make sure we subscribe because of cat
        bers = status.builderAdded("builder2", self.tempdir, category="debug")
        self.assertEquals(len(bers.watchers), 1)
        self.assertEquals(len(self.messages), 1)
        m = self.messages.pop()
        self.assertEquals(m, "builder builder2 added")

        # finish a failing build and see message
        self._finishBuild(bers, builder.FAILURE, ["bob"])
        self.assertEquals(len(self.messages), 1)

        m = self.messages.pop()
        m2 = words._DEFAULT_BLAMES[0] % "bob"
        self.failUnless(m.startswith(m2))
        self.assertEqual(m.find(" even more."), -1)

        # finish another failing build and see message complaining further
        self._finishBuild(bers, builder.FAILURE, ["moe"])
        self.assertEqual(len(self.messages), 1)

        m = self.messages.pop()
        m2 = words._DEFAULT_BLAMES[0] % "moe"
        self.failUnless(m.startswith(m2))
        self.failIfEqual(m.find(" even more."), -1)

        # finish a successful build and get praised
        self._finishBuild(bers, builder.SUCCESS, ["curly"])
        self.assertEqual(len(self.messages), 1)

        m = self.messages.pop()
        m2 = words._DEFAULT_PRAISES[0] % "curly"
        self.failUnless(m.startswith(m2))
        return
